/*
 * gtk2_options.c
 *
 * this file contains the options dialog for the GTK2 control
 *
 * Copyright (C) 2003,04 Karl H. Beckers, Frankfurt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "../config.h" //autoconf output

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Intrinsic.h>
#ifdef HAVE_LIBZ
#include <zlib.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

// check if we need all of these
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <pango/pango-layout.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include "led_meter.h"



#include "job.h"
#include "capture.h"
#include "app_data.h"
#include "util.h"
#include "codecs.h"
#include "gtk2_support.h"
#include "gtk2_control.h"
#include "gtk2_warning.h"
#include "gtk2_fileselector.h"

//#define OPS_FILE        ".xvidcaprc"


/*
 * this is convenience code created by glade ...
 *
 */
#define GLADE_HOOKUP_OBJECT(component, widget, name) \
    g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)
#define GLADE_HOOKUP_OBJECT_NO_REF(component, widget, name) \
    g_object_set_data (G_OBJECT (component), name, widget)

    
/* 
 * these are global 
 */
GtkWidget *dialog1;


/*
 * include some globals from gtk2_control.c
 *
 */
extern GtkWidget *window1;
extern GtkWidget *filename;
extern GtkWidget *MI_continue;
extern GtkWidget *previous;
extern GtkWidget *next;
extern GtkTooltips *tooltips;
extern xvCodec tCodecs[NUMCODECS];
extern xvFFormat tFFormats[NUMCAPS];
extern xvAuCodec tAuCodecs[NUMAUCODECS];
extern AppData *app;


/*
 * file globals (statics)
 *
 */
static GtkWidget *warning;
static AppData pref_app;
static GtkWidget *cap_mouse_p_black_radio;
static GtkWidget *cap_mouse_p_white_radio;
static GtkWidget *sf_format_combo;
static GtkWidget *sf_format_auto_check;
static GtkWidget *mf_format_combo;
static GtkWidget *mf_format_auto_check;
static char* format_combo_entries[NUMCAPS];
static GtkWidget *mf_codec_auto_check;
static GtkWidget *mf_codec_combo;
static GList *mf_codec_entries;
static GtkWidget *mf_audio_codec_auto_check;
static GtkWidget *mf_audio_sample_rate_spinbutton;
static GtkWidget *mf_audio_bit_rate_spinbutton;
static GtkWidget *mf_audio_in_label;
static GtkWidget *mf_audio_sample_rate_label;
static GtkWidget *mf_audio_bit_rate_label;
static GtkWidget *mf_audio_channels_label;
static GtkWidget *mf_audio_channels_hscale;
static GtkWidget *mf_audio_in_select_button;
static GtkWidget *mf_audio_in_entry;
static GtkWidget *mf_fps_widget;
static int mf_fps_widget_save_old_codec = -1;
static GtkWidget *table6;
static GtkWidget *cmd_help_entry;
static GtkWidget *sf_cmd_encode_entry;
static GtkWidget *sf_cmd_play_entry;
static GtkWidget *sf_cmd_edit_entry;
static GtkWidget *mf_cmd_play_entry;
static GtkWidget *mf_cmd_encode_entry;
static GtkWidget *mf_cmd_edit_entry;
static GtkWidget *single_frame_radio;
static GtkWidget *cap_mouse_p_check;
static GtkWidget *save_cap_geo_check;
static GtkWidget *use_shm_check;
static GtkWidget *autocont_check;
static GtkWidget *sf_file_name_entry;
static GtkWidget *sf_fps_widget;
static GtkWidget *sf_quality_hscale;
static GtkWidget *sf_max_time_spinbutton;
static GtkWidget *sf_max_frames_spinbutton;
static GtkWidget *sf_start_no_spinbutton;
static GtkWidget *sf_compress_hscale;
static GtkWidget *sf_step_spinbutton;
static GtkWidget *mf_file_name_entry;
static GtkWidget *mf_max_time_spinbutton;
static GtkWidget *mf_max_frames_spinbutton;
static GtkWidget *mf_compress_hscale;
static GtkWidget *mf_quality_hscale;
static GtkWidget *mf_audio_check;
static GList *sf_format_items = NULL;
static GtkWidget *mf_audio_codec_combo;
static GtkWidget *mf_audio_codec_label;
static GList *mf_audio_codec_entries = NULL;

static int sf_t_format, mf_t_format;
static int sf_t_codec, mf_t_codec;
static int sf_t_au_codec, mf_t_au_codec;
static int OK_attempts = 0;
static xvErrorListItem *errors_after_cli = NULL;


// first callbacks here ...
//
//

static void read_app_data_from_pref_gui ( AppData *lapp ) {
    // general tab
    // default capture mode
    lapp->default_mode = ((gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( single_frame_radio) ))?0:1);
    // capture mouse pointer
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( cap_mouse_p_check ) ) ) {
        if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( cap_mouse_p_white_radio ) ) ) {
            lapp->mouseWanted = 1;
        } else {
            lapp->mouseWanted = 2;
        }
    } else {
        lapp->mouseWanted = 0;
    }
    // save geometry
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( save_cap_geo_check ) ) ) {
       lapp->flags |= FLG_SAVE_GEOMETRY;
    } else {
        lapp->flags &= ~FLG_SAVE_GEOMETRY;
    }
    // use shared-memory
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( use_shm_check ) ) ) {
        lapp->flags |= FLG_USE_SHM;
    } else {
        lapp->flags &= ~FLG_USE_SHM;
    }
    // autocontinue
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( autocont_check ) ) ) {
        lapp->flags |= FLG_AUTO_CONTINUE;
    } else {
        lapp->flags &= ~FLG_AUTO_CONTINUE;
    }

    // sf
    // file name
    lapp->single_frame.file = strdup((char*) gtk_entry_get_text(GTK_ENTRY(sf_file_name_entry)));
    // file format
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( sf_format_auto_check ) ) ) {
        lapp->single_frame.target = 0;
    } else {
        char *selected_format = (char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(sf_format_combo)->entry ) );
        int n, m = -1;
        
        for ( n = 0; n < g_list_length(sf_format_items) && m < 0; n++) {
            char *format = g_list_nth_data(sf_format_items, n);
            if ( strcasecmp(format, selected_format)==0 ) m = n;
//            printf("%s = %s ?\n", format, selected_format);
        } 
        
        lapp->single_frame.target = ++n;
//        printf("found new target: %i\n", n);
    }
    // codec is always 0 for sf
    lapp->single_frame.targetCodec = 0;
    // au_targetCodec is always 0 for sf
    lapp->single_frame.au_targetCodec = 0;
    // fps
    lapp->single_frame.fps = (int) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GTK_RANGE (sf_fps_widget)->adjustment)) * 100);
    // quality
    lapp->single_frame.quality = gtk_adjustment_get_value (GTK_ADJUSTMENT (GTK_RANGE (sf_quality_hscale)->adjustment));
    // max time
    lapp->single_frame.time = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (sf_max_time_spinbutton));
    // max frames
    lapp->single_frame.frames = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (sf_max_frames_spinbutton));
    // compress
    lapp->single_frame.compress = gtk_adjustment_get_value (GTK_ADJUSTMENT (GTK_RANGE (sf_compress_hscale)->adjustment));
    // start_no
    lapp->single_frame.start_no = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (sf_start_no_spinbutton));
    // step
    lapp->single_frame.step = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (sf_step_spinbutton));

    // mf
    // file name
    lapp->multi_frame.file = strdup((char*) gtk_entry_get_text(GTK_ENTRY(mf_file_name_entry)));
    // format
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( mf_format_auto_check ) ) ) {
        lapp->multi_frame.target = 0;
    } else {
        char *selected_format = (char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_format_combo)->entry ) );
        int i, n = -1;
        
        for ( i = 0; i < NUMCAPS && n < 0; i++ ) {
            if ( strcasecmp(format_combo_entries[i],selected_format)==0 ) n = i;
        }
        lapp->multi_frame.target = ++n;
    }
    // codec
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( mf_codec_auto_check ) ) ) {
        lapp->multi_frame.targetCodec = 0;
    } else {
        char *selected_codec = (char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_codec_combo)->entry ) );
        int a, codec = -1;

        for (a = 0; a < NUMCODECS && codec < 0; a++ ) {
            if ( strcasecmp(selected_codec, tCodecs[a].longname )==0) 
                codec = a;
        }
        lapp->multi_frame.targetCodec = codec;
    }
    // au_targetCodec
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( mf_audio_codec_auto_check ) ) ) {
        lapp->multi_frame.au_targetCodec = 0;
    } else {
        char *selected_aucodec = (char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_audio_codec_combo)->entry ) );
        int a, aucodec = -1;

        for (a = 0; a < NUMAUCODECS && aucodec < 0; a++ ) {
#ifdef DEBUG
            printf("gtk_options: selected_aucodec: %s - a: %i - tAuCodec[a].name: %s \n", selected_aucodec,
                    a, tAuCodecs[a].name );
#endif // DEBUG
            if ( strcasecmp(selected_aucodec, tAuCodecs[a].name )==0) 
                aucodec = a;
        }
        lapp->multi_frame.au_targetCodec = aucodec;
#ifdef DEBUG
        printf("read_app_data_from_pref_gui: saving audio codec: %i \n", lapp->multi_frame.au_targetCodec);
#endif // DEBUG
    }
    // fps
    {
        gchar *path, *widget_class;
        gtk_widget_class_path (mf_fps_widget, NULL, &path, NULL);
#ifdef DEBUG
        printf("gtk2_options: mf_fps_widget has this classpath: %s\n", (char*) path);
#endif // DEBUG
        widget_class = rindex(path, '.');
        widget_class++;
#ifdef DEBUG
        printf("gtk2_options: mf_fps_widget is of class: %s\n", (char*) widget_class);
#endif // DEBUG
        
        if (strcasecmp(widget_class, "gtkhscale")==0) {
            lapp->multi_frame.fps = (int) (gtk_range_get_value ( GTK_RANGE(mf_fps_widget) ) * 100);
        } else if (strcasecmp(widget_class, "gtkcombo")==0) {
            lapp->multi_frame.fps = get_int_from_float_string((char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_fps_widget)->entry ) ));
        } else {
            fprintf(stderr, "gtk2_options: unable to determine type of mf_fps_widget, please file a bug!\n");
            exit (1);
        }
//        printf("gtk2_options: fps = %i\n", lapp->multi_frame.fps);
    }
    // max time
    lapp->multi_frame.time = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (mf_max_time_spinbutton));
    // max frames
    lapp->multi_frame.frames = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (mf_max_frames_spinbutton));
    // compress
    lapp->multi_frame.compress = gtk_adjustment_get_value (GTK_ADJUSTMENT (GTK_RANGE (mf_compress_hscale)->adjustment));
    // quality
    lapp->multi_frame.quality = gtk_adjustment_get_value (GTK_ADJUSTMENT (GTK_RANGE (mf_quality_hscale)->adjustment));

    // mf audio settings
    // audio wanted
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( mf_audio_check ) ) ) {
        lapp->multi_frame.audioWanted = 1;
    } else {
        lapp->multi_frame.audioWanted = 0;
    }
    // sample rate
    lapp->multi_frame.sndrate = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (mf_audio_sample_rate_spinbutton));
    // bit rate
    lapp->multi_frame.sndsize = (int) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (mf_audio_bit_rate_spinbutton));
    // audio in
    lapp->snddev = strdup((char*) gtk_entry_get_text(GTK_ENTRY(mf_audio_in_entry)));
    // audio channels
    lapp->multi_frame.sndchannels = gtk_adjustment_get_value (GTK_ADJUSTMENT (GTK_RANGE (mf_audio_channels_hscale)->adjustment));
    
    // commands
    // help cmd
    lapp->help_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(cmd_help_entry)));
    // sf commands
    // sf playback command
    lapp->single_frame.play_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(sf_cmd_play_entry)));
    // sf encoding command
    lapp->single_frame.video_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(sf_cmd_encode_entry)));
    // sf edit command
    lapp->single_frame.edit_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(sf_cmd_edit_entry)));
    // mf commands
    // mf playback command
    lapp->multi_frame.play_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(mf_cmd_play_entry)));
    // mf encoding command
    lapp->multi_frame.video_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(mf_cmd_encode_entry)));
    // mf edit command
    lapp->multi_frame.edit_cmd = strdup((char*) gtk_entry_get_text(GTK_ENTRY(mf_cmd_edit_entry)));
    
}


void preferences_submit () {
    Display *mydisplay;
    Window root; //, topwin;
    XWindowAttributes win_attr;

    app_data_copy ( app, &pref_app );     
    
    // Get X display and Window Attributes
    // notice, here we're still talking app->flags
    if (! ( app->flags & FLG_NOGUI ) ) {
        mydisplay = GDK_DRAWABLE_XDISPLAY( GTK_WIDGET(window1)->window );
    } else {
        mydisplay = XOpenDisplay(NULL);
    }
    root = RootWindow(mydisplay, DefaultScreen(mydisplay));
    if (!XGetWindowAttributes(mydisplay, root, &win_attr))
        perror("Can't get window attributes!\n");
    job_set_from_app_data( app, mydisplay, win_attr );
    // validate the job parameters
    job_validate ();
    
    // set controls active/inactive/sensitive/insensitive according to curren options
    reset_gui_according_to_current_prefs();
   
    if (xv_errors_delete_list (errors_after_cli)) {
        fprintf(stderr, "gtk2_options: Unrecoverable error while freeing error list, please contact the xvidcap project.\n");
        exit (1);
    }
    
    // reset attempts so warnings will be shown again next time ...
    OK_attempts = 0;
    
    gtk_widget_destroy(dialog1);
}


void reset_OK_attempts() {
    OK_attempts = 0;
}


void dialog1_doOk () {
    int count_non_info_messages, rc;


    count_non_info_messages = 0;
    rc = 0;
    errors_after_cli = app_data_validate (&pref_app, 0, &rc);
    if (rc == -1) {
        fprintf (stderr, "gtk2_options: Unrecoverable error while validating options, please contact the xvidcap project.\n");
        exit (1);
    }
        
    // warning dialog
    if ( errors_after_cli != NULL ) {
        xvErrorListItem *err;

        err = errors_after_cli;
        for (;err != NULL; err = err->next) {
            if ( err->err->type != XV_ERR_INFO ) count_non_info_messages++;
        }

        if ( count_non_info_messages > 0 || ( app->flags & FLG_RUN_VERBOSE && OK_attempts == 0 ) ) {
            warning = create_warning_with_errors (errors_after_cli, 0);
//          printf("gtk2_options: pointer to errors_after_cli: %p - rc: %i\n", errors_after_cli, rc);
        OK_attempts++;

        } else {
            preferences_submit();
        }
    } else {
        preferences_submit ();   
    }
        
/*    if ( count_non_info_messages > 0 ) {
        fprintf (stderr, "gtk2_options: Can't resolve all conflicts in input data. Please double-check your input.\n");
        exit (1);
    } */
    
} 


static void
doHelp () {
    if ( pref_app.help_cmd != NULL ) system((char*) pref_app.help_cmd );
}    


static void
on_dialog1_response (GtkDialog *dialog, gint response_id, gpointer user_data) {
    Job *job = job_ptr ();
    
    //printf("response id : %i \n", response_id);
    switch (response_id) {
        case GTK_RESPONSE_OK:
#ifdef DEBUG
            printf("gtk2_options: (pre) filename: %s - target: %i - targetCodec: %i - audioCodec: %i \n", pref_app.multi_frame.file,
            pref_app.multi_frame.target, pref_app.multi_frame.targetCodec, pref_app.multi_frame.au_targetCodec);
#endif // DEBUG

            // need to read pref_data from gui
            read_app_data_from_pref_gui ( &pref_app );
#ifdef DEBUG
            printf("gtk2_options: (post) filename: %s - target: %i - targetCodec: %i - audioCodec: %i \n", pref_app.multi_frame.file,
            pref_app.multi_frame.target, pref_app.multi_frame.targetCodec, pref_app.multi_frame.au_targetCodec);
#endif // DEBUG
        
            dialog1_doOk ();
            GtkChangeLabel (job->pic_no);
            break;
        case GTK_RESPONSE_CANCEL:
            gtk_widget_destroy(dialog1);
            break;
        case GTK_RESPONSE_HELP:
            doHelp();
        default:
            break;
    }
}


static gboolean
on_dialog1_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
    return FALSE;
}

static void
on_dialog1_close (GtkDialog *dialog, gpointer user_data) {
    // empty as yet ...
}

static void
on_dialog1_destroy (GtkButton *button, gpointer user_data) {
    // empty as yet ...
}

static void
on_cap_mouse_p_check_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
    if ( gtk_toggle_button_get_active ( togglebutton ) ) {
        gtk_widget_set_sensitive ( cap_mouse_p_black_radio, TRUE );
        gtk_widget_set_sensitive ( cap_mouse_p_white_radio, TRUE );
    } else {
        gtk_widget_set_sensitive ( cap_mouse_p_black_radio, FALSE );
        gtk_widget_set_sensitive ( cap_mouse_p_white_radio, FALSE );
    }
    
}

static void
on_sf_format_auto_check_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
    if ( gtk_toggle_button_get_active ( togglebutton ) ) {
        gtk_widget_set_sensitive ( sf_format_combo, FALSE );
    } else {
        gtk_widget_set_sensitive ( sf_format_combo, TRUE );
    }
}

static void
on_sf_file_name_entry_changed (GtkEntry *entry, gpointer user_data) {
    int i;

    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sf_format_auto_check) ) ) {
        i = xv_codec_get_target_from_filename ((char*) gtk_entry_get_text ( GTK_ENTRY (entry) ));
        if ( i > 0 && i < CAP_FFM ) 
            gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(sf_format_combo)->entry), strdup(format_combo_entries[i-1]));
    }
}

static void
on_mf_format_auto_check_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
    if ( gtk_toggle_button_get_active ( togglebutton ) ) {
        gtk_widget_set_sensitive ( mf_format_combo, FALSE );
    } else {
        gtk_widget_set_sensitive ( mf_format_combo, TRUE );
    }
}

static void
on_mf_file_name_entry_changed (GtkEntry *entry, gpointer user_data) {
    int i;

    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(mf_format_auto_check) ) ) {
        i = xv_codec_get_target_from_filename ((char*) gtk_entry_get_text ( GTK_ENTRY (entry) ));
        if ( i >= CAP_FFM ) 
            gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_format_combo)->entry), strdup(format_combo_entries[i-1]));
    }
}

static void
on_mf_codec_auto_check_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
    if ( gtk_toggle_button_get_active ( togglebutton ) ) {
        gtk_widget_set_sensitive ( mf_codec_combo, FALSE );
    } else {
        gtk_widget_set_sensitive ( mf_codec_combo, TRUE );
    }
}

static void
on_mf_audio_codec_auto_check_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
    if ( gtk_toggle_button_get_active ( togglebutton ) ) {
        gtk_widget_set_sensitive ( mf_audio_codec_combo, FALSE );
    } else {
        gtk_widget_set_sensitive ( mf_audio_codec_combo, TRUE );
    }
}

static void
mf_codec_combo_set_contents_from_format (int format) {
    gchar *utf8text;
    GError *error = NULL;
    char *codec = xv_next_element(tFFormats[format].allowed_vid_codecs);
    mf_codec_entries = NULL;
        
    do {
        int i, j = -1;
        for ( i = 1; i < NUMCODECS && j < 0; i++) {
            if ( codec != NULL && strcasecmp(codec, tCodecs[i].name )==0 ) {
                utf8text = g_convert ((gchar*) tCodecs[i].longname, strlen(tCodecs[i].longname), "UTF-8", "ISO-8859-1", NULL, NULL, &error);
                if ( error != NULL ) printf("gtk2-options: couldn't convert codec name to utf8\n");
                mf_codec_entries = g_list_append ( mf_codec_entries, strdup((char*) utf8text));
                j = i;
            }
        }
        codec = xv_next_element(NULL);
    } while ( codec != NULL );
    
    if ( mf_codec_entries != NULL ) {
        gtk_combo_set_popdown_strings (GTK_COMBO (mf_codec_combo), mf_codec_entries);
        gtk_widget_set_sensitive ( mf_codec_auto_check, TRUE );
        if ( ! ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(mf_codec_auto_check) ) ) )
            gtk_widget_set_sensitive ( mf_codec_combo, TRUE );
    } else {
        mf_codec_entries = g_list_append ( mf_codec_entries, "none");
        gtk_combo_set_popdown_strings (GTK_COMBO (mf_codec_combo), mf_codec_entries);
        gtk_widget_set_sensitive ( mf_codec_auto_check, FALSE );
        gtk_widget_set_sensitive ( mf_codec_combo, FALSE );
    }
    
}

static void
mf_audio_codec_combo_set_contents_from_format (int format) {
    char *aucodec = xv_next_element(tFFormats[format].allowed_au_codecs);
    gchar *utf8text;
    GError *error = NULL;
    mf_audio_codec_entries = NULL;
    
    do {
        int i, j = -1;
        for ( i = 1; i < NUMAUCODECS && j < 0; i++) {
            if ( aucodec != NULL && strcasecmp(aucodec, tAuCodecs[i].name )==0 ) {
                utf8text = g_convert ((gchar*) tAuCodecs[i].name, 
                    strlen(tAuCodecs[i].name), "UTF-8", "ISO-8859-1", NULL, NULL, &error);
                if ( error != NULL ) printf("gtk2-options: couldn't convert audio codec name to utf8\n");
                mf_audio_codec_entries = g_list_append ( mf_audio_codec_entries, strdup((char*) utf8text));
                j = i;
            }
        }
        aucodec = xv_next_element(NULL);
    } while ( aucodec != NULL );
    
    if ( mf_audio_codec_entries != NULL ) {
        gtk_combo_set_popdown_strings (GTK_COMBO (mf_audio_codec_combo), mf_audio_codec_entries);
        gtk_widget_set_sensitive ( mf_audio_codec_auto_check, TRUE );
        if ( ! ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(mf_audio_codec_auto_check) ) ) )
            gtk_widget_set_sensitive ( mf_audio_codec_combo, TRUE );
    } else {
        mf_audio_codec_entries = g_list_append ( mf_audio_codec_entries, "none");
        gtk_combo_set_popdown_strings (GTK_COMBO (mf_audio_codec_combo), mf_audio_codec_entries);
        gtk_widget_set_sensitive ( mf_audio_codec_auto_check, FALSE );
        gtk_widget_set_sensitive ( mf_audio_codec_combo, FALSE );
    }

}

static int 
mf_format_combo_get_target_from_text (char *text) {
    int a, b = -1;
    
    for ( a = (CAP_FFM-1); a < (NUMCAPS-1) && b < 0; a++ ) {
        if (strcasecmp(format_combo_entries[a], text)==0) b = a;
    }
    return (b + 1);
}


static void
on_mf_format_combo_changed (GtkEntry *entry, gpointer user_data) {
    char *format_selected = (char*) gtk_entry_get_text( entry );
    int a;
#ifdef DEBUG
    printf("gtk_options: reading '%s' from format combo box ... ", format_selected);
    if ( ! strlen(format_selected) > 0 ) {
        printf("returning\n");
        return;
    } else printf("continuing\n");
#endif // DEBUG
    
    a = mf_format_combo_get_target_from_text (format_selected);
//    printf("format selected: %s - number: %i\n", format_selected, a);
    if (a > 0) {
        char *old_selected_codec = strdup((char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_codec_combo)->entry ) ));
        char *old_selected_aucodec = strdup((char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_audio_codec_combo)->entry ) ));
        int n, m = -1;
        mf_codec_combo_set_contents_from_format (a);    

        for ( n = 0; n < g_list_length(mf_codec_entries) && m < 0; n++) {
            char *codec = g_list_nth_data(mf_codec_entries, n);
            if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(mf_codec_auto_check) ) ) {
                if ( strcasecmp(codec, tCodecs[tFFormats[a].def_vid_codec].longname)==0) {
                    m = n;
                    gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_codec_combo)->entry), strdup(g_list_nth_data(mf_codec_entries, n)));
                }
            } else if ( strlen( old_selected_codec ) > 0 ) {
                if ( strcasecmp(codec, old_selected_codec)==0) {
                    m = n;
                    gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_codec_combo)->entry), strdup(old_selected_codec));
                }
            } else if ( strcasecmp( codec, tCodecs[pref_app.multi_frame.targetCodec].longname)==0 ) {
                m = n;
                gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_codec_combo)->entry), strdup(g_list_nth_data(mf_codec_entries, n)));
            }
        } 
        
        mf_audio_codec_combo_set_contents_from_format (a);
        m = -1;
        
        for ( n = 0; n < g_list_length(mf_audio_codec_entries) && m < 0; n++) {
            char *aucodec = g_list_nth_data(mf_audio_codec_entries, n);
            if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(mf_audio_codec_auto_check) ) ) {
                if ( strcasecmp(aucodec, tAuCodecs[tFFormats[a].def_au_codec].name)==0) {
                    m = n;
                    gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_audio_codec_combo)->entry), strdup(g_list_nth_data(mf_audio_codec_entries, n)));
                }
            } else if ( strlen( old_selected_aucodec ) > 0 ) {
                if ( strcasecmp(aucodec, old_selected_aucodec)==0) {
                    m = n;
                    gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_audio_codec_combo)->entry), strdup(old_selected_aucodec));
                }
            } else if ( strcasecmp( aucodec, tAuCodecs[pref_app.multi_frame.au_targetCodec].name)==0 ) {
                m = n;
                gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_audio_codec_combo)->entry), strdup(g_list_nth_data(mf_audio_codec_entries, n)));
            } 
        } 
        
    }
}


static void
on_mf_audio_check_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
    if ( ! (gtk_toggle_button_get_active ( togglebutton ) )) {
        gtk_widget_set_sensitive ( mf_audio_sample_rate_spinbutton, FALSE );
        gtk_widget_set_sensitive ( mf_audio_bit_rate_spinbutton, FALSE );
        gtk_widget_set_sensitive ( mf_audio_in_label, FALSE );
        gtk_widget_set_sensitive ( mf_audio_sample_rate_label, FALSE );
        gtk_widget_set_sensitive ( mf_audio_bit_rate_label, FALSE );
        gtk_widget_set_sensitive ( mf_audio_channels_label, FALSE );
        gtk_widget_set_sensitive ( mf_audio_channels_hscale, FALSE );
//        gtk_widget_set_sensitive ( mf_audio_in_select_button, FALSE );
        gtk_widget_set_sensitive ( mf_audio_in_entry, FALSE );
        gtk_widget_set_sensitive ( mf_audio_codec_auto_check, FALSE );
        gtk_widget_set_sensitive ( mf_audio_codec_combo, FALSE );
        gtk_widget_set_sensitive ( mf_audio_codec_label, FALSE );
    } else {
        gtk_widget_set_sensitive ( mf_audio_sample_rate_spinbutton, TRUE );
        gtk_widget_set_sensitive ( mf_audio_bit_rate_spinbutton, TRUE );
        gtk_widget_set_sensitive ( mf_audio_in_label, TRUE );
        gtk_widget_set_sensitive ( mf_audio_sample_rate_label, TRUE );
        gtk_widget_set_sensitive ( mf_audio_bit_rate_label, TRUE );
        gtk_widget_set_sensitive ( mf_audio_channels_label, TRUE );
        gtk_widget_set_sensitive ( mf_audio_channels_hscale, TRUE );
//        gtk_widget_set_sensitive ( mf_audio_in_select_button, TRUE );
        gtk_widget_set_sensitive ( mf_audio_in_entry, TRUE );
        gtk_widget_set_sensitive ( mf_audio_codec_auto_check, TRUE );
        gtk_widget_set_sensitive ( mf_audio_codec_combo, TRUE );
        gtk_widget_set_sensitive ( mf_audio_codec_label, TRUE );
    }
}


static void
set_mf_fps_widget_from_codec(int codec) {
    int curr_fps_val = 0;
    
    if ( codec == mf_fps_widget_save_old_codec ) return;
    // store current fps
    {
        gchar *path, *widget_class;
        gtk_widget_class_path (mf_fps_widget, NULL, &path, NULL);
#ifdef DEBUG
        printf("gtk2_options: mf_fps_widget has this classpath: %s\n", (char*) path);
#endif // DEBUG
        widget_class = rindex(path, '.');
        widget_class++;
#ifdef DEBUG
        printf("gtk2_options: mf_fps_widget is of class: %s\n", (char*) widget_class);
#endif // DEBUG
        
        if (strcasecmp(widget_class, "gtkhscale")==0) {
            curr_fps_val = (int) (gtk_range_get_value ( GTK_RANGE(mf_fps_widget) ) * 100);
        } else if (strcasecmp(widget_class, "gtkcombo")==0) {
            curr_fps_val = get_int_from_float_string((char*) gtk_entry_get_text( GTK_ENTRY( GTK_COMBO(mf_fps_widget)->entry ) ));
        } else {
            fprintf(stderr, "gtk2_options: unable to determine type of mf_fps_widget, please file a bug!\n");
            exit (1);
        }
//        printf("gtk2_options: fps = %i\n", lapp->multi_frame.fps);
    }
    
    if ( tCodecs[codec].allowed_fps != NULL && tCodecs[codec].allowed_fps_ranges == NULL ) {
        // we only have a number of non-continuous fps allowed for this codec
        GList *mf_fps_entries = NULL;
        GtkRequisition size;
        char *element, fps_val[256];

        // destroy old widget
        gtk_widget_destroy( mf_fps_widget );
        
        // create new widget
        mf_fps_widget = gtk_combo_new ();
        gtk_combo_set_value_in_list (GTK_COMBO (mf_fps_widget), TRUE, FALSE);
        g_object_set_data (G_OBJECT (GTK_COMBO (mf_fps_widget)->popwin),
                     "GladeParentKey", mf_fps_widget);

        element = xv_next_element ( tCodecs[codec].allowed_fps );
        do {
            // the following is needed for correct handling of locale's decimal point
            char lelement[256];
            int fps = get_int_from_float_string(element);
            sprintf(lelement, "%.2f", ((float) (fps) / 100) );
            
            mf_fps_entries = g_list_append ( mf_fps_entries, strdup(lelement) );
            element = xv_next_element ( NULL );
        } while ( element != NULL );
        gtk_combo_set_popdown_strings (GTK_COMBO (mf_fps_widget), mf_fps_entries);
        gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(mf_fps_widget)->entry), FALSE);
        // resize widget
        // otherwise the combobox after a slider will be to wide ... need to 
        // specify height explicitly or combobox arrow  will be too small
        gtk_widget_size_request(mf_fps_widget, &size);
        gtk_widget_set_size_request (mf_fps_widget, 0, size.height);

        // check current value is allowed or reset
        sprintf(fps_val, "%.2f", ((float) curr_fps_val/100) );
        if ( g_list_find ( mf_fps_entries, fps_val ) == NULL ) {
            sprintf(fps_val, "%.2f", ((float) tCodecs[codec].def_fps) );
            gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_fps_widget)->entry), strdup(fps_val) );
        } else {
            gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(mf_fps_widget)->entry), strdup(fps_val));
        }
        // show
        gtk_widget_show (mf_fps_widget);
        gtk_table_attach (GTK_TABLE (table6), mf_fps_widget, 1, 2, 0, 1,
            (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
            (GtkAttachOptions) (GTK_FILL), 0, 0);
        
    } else if ( tCodecs[codec].allowed_fps_ranges != NULL && tCodecs[codec].allowed_fps == NULL &&
            xv_num_elements(tCodecs[codec].allowed_fps_ranges) == 1 ) {
        // we only have exactly one range 
        char* range = strdup( tCodecs[codec].allowed_fps_ranges );
        char* start = strtok( range, "-" );
        char* end = strtok( NULL, "-" );
        
        if (start == NULL || end == NULL ) {
            fprintf(stderr, "gtk2_options: this codec supposedly only has one valid range of fps's, but we can't determin start and end.\n");
            exit (2);
        }

        // destroy the old widget
        gtk_widget_destroy( mf_fps_widget );
        // check current fps value
        if ( curr_fps_val < get_int_from_float_string(start) || 
                curr_fps_val > get_int_from_float_string(end) ) {
                    curr_fps_val = ((int) tCodecs[codec].def_fps * 100);
        }
        // create new widget
        mf_fps_widget = 
            gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new ( ((float) curr_fps_val/100), 
                    ((float)get_int_from_float_string(start)/100), ((float)get_int_from_float_string(end)/100), 0.01, 1, 0)));
        gtk_widget_show (mf_fps_widget);
        gtk_table_attach (GTK_TABLE (table6), mf_fps_widget, 1, 2, 0, 1,
        (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
        (GtkAttachOptions) (GTK_FILL), 0, 0);
        gtk_scale_set_digits (GTK_SCALE (mf_fps_widget), 2);
    } else {
        // we have both
        char *fixed_start, *fixed_end = NULL, *fixed_buf, *start = NULL, *end = NULL;
        char* range = xv_next_element ( tCodecs[codec].allowed_fps_ranges );
        
        if ( range != NULL ) {
            start = strdup( strtok( range, "-" ) );
            end = strdup( strtok( NULL, "-" ) );
        }
        range = xv_next_element ( NULL );
        while (range != NULL) {
            end = strtok( range, "-" );
            end = strdup( strtok( NULL, "-" ) );
            range = xv_next_element ( NULL );
        }
        fixed_start = xv_next_element( tCodecs[codec].allowed_fps );
        if ( strtod(fixed_start,NULL) < strtod(start,NULL) ) start = fixed_start;
        fixed_buf = xv_next_element( NULL );
        while ( fixed_buf != NULL ) {
            fixed_end = strdup ( fixed_buf );
            fixed_buf = xv_next_element( NULL );
        }
        if ( strtod(fixed_end,NULL) > strtod(end,NULL) ) end = fixed_end;
        
        // destroy old widget
        gtk_widget_destroy( mf_fps_widget );
        // check current fps value
        if ( curr_fps_val < get_int_from_float_string(start) || 
                curr_fps_val > get_int_from_float_string(end) ) {
                    curr_fps_val = ((int) tCodecs[codec].def_fps * 100);
        }
        mf_fps_widget = 
            gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new ( ((float)curr_fps_val/100), 
                    ((float)get_int_from_float_string(start)/100), ((float)get_int_from_float_string(end)/100), 0.01, 1, 0)));
        gtk_widget_show (mf_fps_widget);
        gtk_table_attach (GTK_TABLE (table6), mf_fps_widget, 1, 2, 0, 1,
        (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
        (GtkAttachOptions) (GTK_FILL), 0, 0);
        gtk_scale_set_digits (GTK_SCALE (mf_fps_widget), 2);

    }
    
    mf_fps_widget_save_old_codec = codec;    
}

static void
on_mf_codec_combo_changed (GtkEntry *entry, gpointer user_data) {
    int a, codec = -1;
    char* codec_selected = (char*) gtk_entry_get_text(entry);
    
    for (a = 0; a < NUMCODECS && codec < 0; a++ ) {
        if ( strcasecmp(codec_selected, tCodecs[a].longname )==0) 
            codec = a;
    }
    
    if (codec > 0) set_mf_fps_widget_from_codec(codec);

}

static void
on_cmd_help_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_help_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( cmd_help_entry ) );
    if ( curr_help_cmd != NULL ) system((char*) curr_help_cmd );
}

static void
on_sf_cmd_play_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_play_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( sf_cmd_play_entry ) );
    xv_command_execute (curr_play_cmd, 1, 0, pref_app.single_frame.file, pref_app.single_frame.start_no, (pref_app.single_frame.start_no+1), 
            pref_app.cap_width, pref_app.cap_height, pref_app.single_frame.fps, (int) (1000 / (pref_app.single_frame.fps/100)) );
}

static void
on_sf_cmd_encode_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_encode_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( sf_cmd_encode_entry ) );
    xv_command_execute (curr_encode_cmd, 0, 0, pref_app.single_frame.file, pref_app.single_frame.start_no, (pref_app.single_frame.start_no+1), 
            pref_app.cap_width, pref_app.cap_height, pref_app.single_frame.fps, (int) (1000 / (pref_app.single_frame.fps/100)) );
}

static void
on_sf_cmd_edit_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_edit_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( sf_cmd_edit_entry ) );
    xv_command_execute (curr_edit_cmd, 2, pref_app.single_frame.start_no, pref_app.single_frame.file, pref_app.single_frame.start_no, (pref_app.single_frame.start_no+1), 
            pref_app.cap_width, pref_app.cap_height, pref_app.single_frame.fps, (int) (1000 / (pref_app.single_frame.fps/100)) );
}

static void
on_mf_cmd_play_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_play_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( mf_cmd_play_entry ) );
    xv_command_execute (curr_play_cmd, 2, 0, pref_app.multi_frame.file, pref_app.multi_frame.start_no, (pref_app.multi_frame.start_no+1), 
            pref_app.cap_width, pref_app.cap_height, pref_app.multi_frame.fps, (int) (1000 / (pref_app.multi_frame.fps/100)) );
}

static void
on_mf_cmd_encode_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_encode_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( mf_cmd_encode_entry ) );
    xv_command_execute (curr_encode_cmd, 2, 0, pref_app.multi_frame.file, pref_app.multi_frame.start_no, (pref_app.multi_frame.start_no+1), 
            pref_app.cap_width, pref_app.cap_height, pref_app.multi_frame.fps, (int) (1000 / (pref_app.multi_frame.fps/100)) );
}

static void
on_mf_cmd_edit_try_button_clicked (GtkButton *button, gpointer user_data) {
    char *curr_edit_cmd = (char*) gtk_entry_get_text ( GTK_ENTRY( mf_cmd_edit_entry ) );
    xv_command_execute (curr_edit_cmd, 2, 0, pref_app.multi_frame.file, pref_app.multi_frame.start_no, (pref_app.multi_frame.start_no+1), 
            pref_app.cap_width, pref_app.cap_height, pref_app.multi_frame.fps, (int) (1000 / (pref_app.multi_frame.fps/100)) );
}

static void
on_sf_file_select_button_clicked (GtkButton *button, gpointer user_data) {
    GtkWidget *filechooserdialog1;
    int result = 0;
    char* got_file_name;

    filechooserdialog1 = create_filechooserdialog1 ("Select Output File");
    result = gtk_dialog_run (GTK_DIALOG (filechooserdialog1));

    if ( result == GTK_RESPONSE_OK) {
            got_file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooserdialog1));
            gtk_entry_set_text( GTK_ENTRY(sf_file_name_entry), 
                strdup(got_file_name) );
//            printf("selected filename: %s\n", got_file_name);
    }
        
    gtk_widget_destroy (filechooserdialog1);
}

static void
on_mf_file_select_button_clicked (GtkButton *button, gpointer user_data) {
    GtkWidget *filechooserdialog1;
    int result = 0;
    char* got_file_name;

    filechooserdialog1 = create_filechooserdialog1 ("Select Output File");
    result = gtk_dialog_run (GTK_DIALOG (filechooserdialog1));

    if ( result == GTK_RESPONSE_OK) {
            got_file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooserdialog1));
            gtk_entry_set_text( GTK_ENTRY(mf_file_name_entry), 
                strdup(got_file_name) );
//            printf("selected filename: %s\n", got_file_name);
    }
        
    gtk_widget_destroy (filechooserdialog1);
}

static void
on_mf_audio_in_select_button_clicked (GtkButton *button, gpointer user_data) {
    GtkWidget *filechooserdialog1;
    int result = 0;
    char* got_file_name;

    filechooserdialog1 = create_filechooserdialog1 ("Select Audio Input Device");
    result = gtk_dialog_run (GTK_DIALOG (filechooserdialog1));

    if ( result == GTK_RESPONSE_OK) {
            got_file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooserdialog1));
            gtk_entry_set_text( GTK_ENTRY(mf_audio_in_entry), 
                strdup(got_file_name) );
//            printf("selected filename: %s\n", got_file_name);
    }
        
    gtk_widget_destroy (filechooserdialog1);
}




GtkWidget*
create_dialog1 (AppData *lapp) {
    GtkWidget *dialog_vbox1;
    GtkWidget *pref_notebook;
    GtkWidget *frame7;
    GtkWidget *table14;
    GtkWidget *def_capt_label;
    GSList *single_frame_radio_group = NULL;
    GSList *cap_mouse_p_black_radio_group = NULL;
    GtkWidget *multi_frame_radio;
    GtkWidget *label103;
    GtkWidget *general_tab_label;
    GtkWidget *frame3;
    GtkWidget *vbox3;
    GtkWidget *table8;
    GtkWidget *sf_file_name_label;
    GtkWidget *sf_file_format_label;
    GtkWidget *sf_file_select_button;
    GtkWidget *table9;
    GtkWidget *sf_fps_label;
    GtkWidget *sf_max_time_label;
    GtkWidget *sf_quality_label;
    GtkWidget *sf_max_frames_label;
    GtkWidget *sf_compress_label;
    GtkObject *sf_max_time_spinbutton_adj;
    GtkObject *sf_max_frames_spinbutton_adj;
    GtkWidget *sf_start_no_label;
    GtkWidget *sf_step_label;
    GtkObject *sf_start_no_spinbutton_adj;
    GtkObject *sf_step_spinbutton_adj;
    GtkWidget *label60;
    GtkWidget *single_tab_label;
    GtkWidget *vbox1;
    GtkWidget *frame1;
    GtkWidget *vbox2;
    GtkWidget *table5;
    GtkWidget *mf_file_name_label;
    GtkWidget *mf_file_format_label;
    GtkWidget *mf_codec_label;
    GtkWidget *mf_file_select_button;
    GList *mf_format_items = NULL;
    GtkWidget *mf_fps_label;
    GtkWidget *mf_max_time_label;
    GtkWidget *mf_max_frames_label;
    GtkWidget *mf_compress_label;
    GtkWidget *mf_quality_label;
    GtkObject *mf_max_time_spinbutton_adj;
    GtkObject *mf_max_frames_spinbutton_adj;
    GtkWidget *label43;
    GtkWidget *frame2;
    GtkWidget *table7;
    GtkObject *mf_audio_sample_rate_spinbutton_adj;
    GtkObject *mf_audio_bit_rate_spinbutton_adj;
    GtkWidget *hbox3;
    GtkWidget *label44;
    GtkWidget *multi_tab_label;
    GtkWidget *vbox5;
    GtkWidget *frame4;
    GtkWidget *hbox1;
    GtkWidget *cmd_help_label;
    GtkWidget *cmd_help_try_button;
    GtkWidget *label93;
    GtkWidget *frame5;
    GtkWidget *table12;
    GtkWidget *sf_cmd_play_label;
    GtkWidget *sf_cmd_encode_label;
    GtkWidget *sf_cmd_edit_label;
    GtkWidget *sf_cmd_play_try_button;
    GtkWidget *sf_cmd_encode_try_button;
    GtkWidget *sf_cmd_edit_try_button;
    GtkWidget *label94;
    GtkWidget *frame6;
    GtkWidget *table13;
    GtkWidget *mf_cmd_play_label;
    GtkWidget *mf_cmd_encode_label;
    GtkWidget *mf_cmd_edit_label;
    GtkWidget *mf_cmd_play_try_button;
    GtkWidget *mf_cmd_encode_try_button;
    GtkWidget *mf_cmd_edit_try_button;
    GtkWidget *label95;
    GtkWidget *label92;
    GtkWidget *dialog_action_area1;
    GtkWidget *helpbutton1;
    GtkWidget *cancelbutton1;
    GtkWidget *okbutton1;
    GtkTooltips *tooltips;
    GtkWidget *hbox4;
    
    // first copy the pass parameters to a temporary structure
    app_data_copy ( &pref_app, lapp);
    sf_t_format = pref_app.single_frame.target;
    if (!sf_t_format) sf_t_format = xv_codec_get_target_from_filename (pref_app.single_frame.file);
    sf_t_codec = pref_app.single_frame.targetCodec;
    if (!sf_t_codec) sf_t_codec = tFFormats[sf_t_format].def_vid_codec;
    sf_t_au_codec = pref_app.single_frame.au_targetCodec;
    if (!sf_t_au_codec) sf_t_au_codec = tFFormats[sf_t_format].def_au_codec;
    
    mf_t_format = pref_app.multi_frame.target;
    if (!mf_t_format) mf_t_format = xv_codec_get_target_from_filename (pref_app.multi_frame.file);
    mf_t_codec = pref_app.multi_frame.targetCodec;
    if (!mf_t_codec) mf_t_codec = tFFormats[mf_t_format].def_vid_codec;
    mf_t_au_codec = pref_app.multi_frame.au_targetCodec;
    if (!mf_t_au_codec) mf_t_au_codec = tFFormats[mf_t_format].def_au_codec;
    
    // prepare the arrays with the entries for the dynamic comboboxes
    {
        int a;
        char buf[256];
        gchar *utf8text;
        GError *error = NULL;
        
        for (a = 1; a < NUMCAPS; a++) {
            sprintf(buf, "%s (%s)", tFFormats[a].longname, xv_next_element(tFFormats[a].extensions) );
            utf8text = g_convert ((gchar*) buf, strlen(buf), "UTF-8", "ISO-8859-1", NULL, NULL, &error);
            if ( error != NULL ) printf("gtk2-options: couldn't convert format name to utf8\n");
            format_combo_entries[a-1] = strdup((char*)utf8text);
        }
    }
    
    // prepare tooltips
    tooltips = gtk_tooltips_new ();
    
    // the main dialog window
    dialog1 = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (dialog1), _ ("Preferences"));
    gtk_window_set_modal (GTK_WINDOW (dialog1), TRUE );
    
    dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
    gtk_widget_show (dialog_vbox1);
    
    pref_notebook = gtk_notebook_new ();
    gtk_widget_show (pref_notebook);
    gtk_box_pack_start (GTK_BOX (dialog_vbox1), pref_notebook, TRUE, TRUE, 0);
    
    // general tab
    frame7 = gtk_frame_new (NULL);
    gtk_widget_show (frame7);
    gtk_container_add (GTK_CONTAINER (pref_notebook), frame7);
    gtk_container_set_border_width (GTK_CONTAINER (frame7), 4);
    
    table14 = gtk_table_new (5, 3, FALSE);
    gtk_widget_show (table14);
    gtk_container_add (GTK_CONTAINER (frame7), table14);
    gtk_container_set_border_width (GTK_CONTAINER (table14), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table14), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table14), 10);
    
    // default capture mode
    def_capt_label = gtk_label_new (_ ("Default Capture Mode:"));
    gtk_widget_show (def_capt_label);
    gtk_table_attach (GTK_TABLE (table14), def_capt_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (def_capt_label), 0, 0.5);
    
    single_frame_radio = gtk_radio_button_new_with_mnemonic (NULL, _ ("Single-Frame"));
    gtk_widget_show (single_frame_radio);
    gtk_table_attach (GTK_TABLE (table14), single_frame_radio, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, single_frame_radio, _ ("Use Single-Frame Capture as default, i.e. if nothing else is explicitly specified."), NULL);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (single_frame_radio), single_frame_radio_group);
    single_frame_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (single_frame_radio));
    
    multi_frame_radio = gtk_radio_button_new_with_mnemonic (NULL, _ ("Multi-Frame"));
    gtk_widget_show (multi_frame_radio);
    gtk_table_attach (GTK_TABLE (table14), multi_frame_radio, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, multi_frame_radio, _ ("Use Multi-Frame Capture as default, i.e. if nothing else is explicitly specified."), NULL);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (multi_frame_radio), single_frame_radio_group);
    single_frame_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (multi_frame_radio));
    
    if ( pref_app.default_mode == 0 ) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (single_frame_radio), TRUE);
    else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (multi_frame_radio), TRUE);
    
    // capture mouse pointer
    cap_mouse_p_check = gtk_check_button_new_with_mnemonic (_ ("Capture _Mouse Pointer"));
    gtk_widget_show (cap_mouse_p_check);
    gtk_table_attach (GTK_TABLE (table14), cap_mouse_p_check, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, cap_mouse_p_check, _ ("Enable Capturing of Mouse Pointer Movement."), NULL);
    
    cap_mouse_p_black_radio = gtk_radio_button_new_with_mnemonic (NULL, _ ("black"));
    gtk_widget_show (cap_mouse_p_black_radio);
    gtk_table_attach (GTK_TABLE (table14), cap_mouse_p_black_radio, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, cap_mouse_p_black_radio, _ ("Capture the Mouse Pointer as a black arrow."), NULL);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (cap_mouse_p_black_radio), cap_mouse_p_black_radio_group);
    cap_mouse_p_black_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (cap_mouse_p_black_radio));
    
    cap_mouse_p_white_radio = gtk_radio_button_new_with_mnemonic (NULL, _ ("white"));
    gtk_widget_show (cap_mouse_p_white_radio);
    gtk_table_attach (GTK_TABLE (table14), cap_mouse_p_white_radio, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, cap_mouse_p_white_radio, _ ("Capture the Mouse Pointer as a white arrow."), NULL);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (cap_mouse_p_white_radio), cap_mouse_p_black_radio_group);
    cap_mouse_p_black_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (cap_mouse_p_white_radio));
    
    if ( pref_app.mouseWanted == 1 ) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cap_mouse_p_white_radio), TRUE);
    else if ( pref_app.mouseWanted == 2 ) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cap_mouse_p_black_radio), TRUE);
    if ( pref_app.mouseWanted == 0 ) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cap_mouse_p_check), FALSE);
        gtk_widget_set_sensitive ( cap_mouse_p_black_radio, FALSE );
        gtk_widget_set_sensitive ( cap_mouse_p_white_radio, FALSE );
    }
    else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cap_mouse_p_check), TRUE);
    
    // save geometry
    save_cap_geo_check = gtk_check_button_new_with_mnemonic (_ ("Save Capture _Geometry"));
    gtk_widget_show (save_cap_geo_check);
    gtk_table_attach (GTK_TABLE (table14), save_cap_geo_check, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, save_cap_geo_check, _ ("Save size and position of the Capture Area to the preferences file."), NULL);
    
    if ((pref_app.flags & FLG_SAVE_GEOMETRY) != 0) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (save_cap_geo_check), TRUE);
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (save_cap_geo_check), FALSE);
    }
    
    // use shared-memory
    use_shm_check = gtk_check_button_new_with_mnemonic (_ ("Use _Shared Memory"));
    gtk_widget_show (use_shm_check);
    gtk_table_attach (GTK_TABLE (table14), use_shm_check, 0, 1, 3, 4,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, use_shm_check, _ ("Use Shared Memory access to X11."), NULL);
    
#ifdef HAVE_SHMAT
    if ( (pref_app.flags & FLG_USE_SHM) != 0 ) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (use_shm_check), TRUE);
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (use_shm_check), FALSE);
    }
#else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (use_shm_check), FALSE);
    gtk_widget_set_sensitive ( use_shm_check, FALSE );
#endif //HAVE_SHMAT

  // autocontinue
  autocont_check = gtk_check_button_new_with_mnemonic (_ ("Auto_continue"));
    gtk_widget_show (autocont_check);
    gtk_table_attach (GTK_TABLE (table14), autocont_check, 0, 1, 4, 5,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, autocont_check, _ ("Enable the autocontinue feature for Multi-Frame Capture."), NULL);
    
    if ((pref_app.flags & FLG_AUTO_CONTINUE) != 0) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autocont_check), TRUE);
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autocont_check), FALSE);
    }
    
    label103 = gtk_label_new (_ ("Miscellaneous Settings"));
    gtk_widget_show (label103);
    gtk_frame_set_label_widget (GTK_FRAME (frame7), label103);
    
    general_tab_label = gtk_label_new_with_mnemonic (_ ("G_eneral"));
    gtk_widget_show (general_tab_label);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (pref_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pref_notebook), 0), general_tab_label);
    
    // single-frame tab
    frame3 = gtk_frame_new (NULL);
    gtk_widget_show (frame3);
    gtk_container_add (GTK_CONTAINER (pref_notebook), frame3);
    gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (pref_notebook), frame3,
    FALSE, FALSE, GTK_PACK_START);
    gtk_container_set_border_width (GTK_CONTAINER (frame3), 4);
    
    vbox3 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox3);
    gtk_container_add (GTK_CONTAINER (frame3), vbox3);
    
    table8 = gtk_table_new (2, 3, FALSE);
    gtk_widget_show (table8);
    gtk_box_pack_start (GTK_BOX (vbox3), table8, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table8), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table8), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table8), 5);
    
    // file name
    sf_file_name_label = gtk_label_new (_ ("File Name:"));
    gtk_widget_show (sf_file_name_label);
    gtk_table_attach (GTK_TABLE (table8), sf_file_name_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_file_name_label), 0, 0.5);
    
    sf_file_select_button = gtk_button_new_with_mnemonic (_ ("select"));
    gtk_widget_show (sf_file_select_button);
    gtk_table_attach (GTK_TABLE (table8), sf_file_select_button, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_file_select_button, _ ("Click to open a file selector."), NULL);
    
    sf_file_name_entry = gtk_entry_new ();
    gtk_widget_show (sf_file_name_entry);
    gtk_table_attach (GTK_TABLE (table8), sf_file_name_entry, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_file_name_entry, _ ("Enter a target filename or pattern here."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( sf_file_name_entry ), strdup(pref_app.single_frame.file) );
    
    // file format
    sf_file_format_label = gtk_label_new (_ ("File Format:"));
    gtk_widget_show (sf_file_format_label);
    gtk_table_attach (GTK_TABLE (table8), sf_file_format_label, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_file_format_label), 0, 0.5);
    
    sf_format_auto_check = gtk_check_button_new_with_mnemonic (_ ("auto"));
    gtk_widget_show (sf_format_auto_check);
    gtk_table_attach (GTK_TABLE (table8), sf_format_auto_check, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_format_auto_check, _ ("Check here for auto-detection of the file format based on the filename."), NULL);
    
    sf_format_combo = gtk_combo_new ();
    gtk_combo_set_value_in_list (GTK_COMBO (sf_format_combo), TRUE, FALSE);
    g_object_set_data (G_OBJECT (GTK_COMBO (sf_format_combo)->popwin),
                     "GladeParentKey", sf_format_combo);
    {
        int a;
        for (a = 0; a < (CAP_FFM -1); a++) {
            sf_format_items = g_list_append ( sf_format_items, format_combo_entries[a] );
        }
    }
    gtk_combo_set_popdown_strings (GTK_COMBO (sf_format_combo), sf_format_items);
    gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(sf_format_combo)->entry), FALSE);
    gtk_widget_show (sf_format_combo);
    gtk_table_attach (GTK_TABLE (table8), sf_format_combo, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_format_combo, _ ("Click and pick a file format from the list of available formats."), NULL);

    if ( pref_app.single_frame.target == 0 ) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sf_format_auto_check), TRUE);
        gtk_widget_set_sensitive ( sf_format_combo, FALSE );
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sf_format_auto_check), FALSE);
        gtk_widget_set_sensitive ( sf_format_combo, TRUE );
    }
    on_sf_file_name_entry_changed (GTK_ENTRY(sf_file_name_entry), NULL);
    
    table9 = gtk_table_new (4, 4, FALSE);
    gtk_widget_show (table9);
    gtk_box_pack_start (GTK_BOX (vbox3), table9, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table9), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table9), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table9), 10);
    
    // fps
    sf_fps_label = gtk_label_new (_ ("Frames per\nSecond:"));
    gtk_widget_show (sf_fps_label);
    gtk_table_attach (GTK_TABLE (table9), sf_fps_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_fps_label), 0, 0.5);
    
    sf_fps_widget = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new ((pref_app.single_frame.fps/100), 0.01, 100, 0.01, 1, 0)));
    gtk_widget_show (sf_fps_widget);
    gtk_table_attach (GTK_TABLE (table9), sf_fps_widget, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (sf_fps_widget), 2);
    
    // quality
    sf_quality_label = gtk_label_new (_ ("Quality:"));
    gtk_widget_show (sf_quality_label);
    gtk_table_attach (GTK_TABLE (table9), sf_quality_label, 0, 1, 3, 4,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_quality_label), 0, 0.5);
    
    sf_quality_hscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (pref_app.single_frame.quality, 0.1, 100, 1, 10, 0)));
    gtk_widget_show (sf_quality_hscale);
    gtk_table_attach (GTK_TABLE (table9), sf_quality_hscale, 1, 2, 3, 4,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (sf_quality_hscale), 0);
    
    // max time
    sf_max_time_label = gtk_label_new (_ ("Maximum\nTime:"));
    gtk_widget_show (sf_max_time_label);
    gtk_table_attach (GTK_TABLE (table9), sf_max_time_label, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_max_time_label), 0, 0.5);

    sf_max_time_spinbutton_adj = gtk_adjustment_new (pref_app.single_frame.time, 0, 1e+08, 1, 10, 10);
    sf_max_time_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (sf_max_time_spinbutton_adj), 1, 0);
    gtk_widget_show (sf_max_time_spinbutton);
    gtk_table_attach (GTK_TABLE (table9), sf_max_time_spinbutton, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_max_time_spinbutton, _ ("Set the maximum time to record in number of seconds. A value of zero makes a capture session run until stopped interactively."), NULL);
    
    // max frames
    sf_max_frames_label = gtk_label_new (_ ("Maximum\nFrames:"));
    gtk_widget_show (sf_max_frames_label);
    gtk_table_attach (GTK_TABLE (table9), sf_max_frames_label, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_max_frames_label), 0, 0.5);

    sf_max_frames_spinbutton_adj = gtk_adjustment_new (pref_app.single_frame.frames, 0, 1e+08, 1, 10, 10);
    sf_max_frames_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (sf_max_frames_spinbutton_adj), 1, 0);
    gtk_widget_show (sf_max_frames_spinbutton);
    gtk_table_attach (GTK_TABLE (table9), sf_max_frames_spinbutton, 3, 4, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_max_frames_spinbutton, _ ("Set the maximum number of frames to record. A value of zero makes a capture session run until stopped interactively."), NULL);

    // compress
    sf_compress_label = gtk_label_new (_ ("Compression\nLevel:"));
    gtk_widget_show (sf_compress_label);
    gtk_table_attach (GTK_TABLE (table9), sf_compress_label, 2, 3, 3, 4,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_compress_label), 0, 0.5);
    
    sf_compress_hscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (pref_app.single_frame.compress, 0, 9, 1, 5, 0)));
    gtk_widget_show (sf_compress_hscale);
    gtk_table_attach (GTK_TABLE (table9), sf_compress_hscale, 3, 4, 3, 4,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (sf_compress_hscale), 0);
    
    // start_no
    sf_start_no_label = gtk_label_new (_ ("Frame Start\nNumber:"));
    gtk_widget_show (sf_start_no_label);
    gtk_table_attach (GTK_TABLE (table9), sf_start_no_label, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_start_no_label), 0, 0.5);

    sf_start_no_spinbutton_adj = gtk_adjustment_new (pref_app.single_frame.start_no, 0, 1e+08, 1, 10, 10);
    sf_start_no_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (sf_start_no_spinbutton_adj), 1, 0);
    gtk_widget_show (sf_start_no_spinbutton);
    gtk_table_attach (GTK_TABLE (table9), sf_start_no_spinbutton, 1, 2, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_start_no_spinbutton, _ ("Start numbering captured frames at the number provided."), NULL);
    
    // step
    sf_step_label = gtk_label_new (_ ("Frame Number\nIncrement:"));
    gtk_widget_show (sf_step_label);
    gtk_table_attach (GTK_TABLE (table9), sf_step_label, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_step_label), 0, 0.5);
    
    sf_step_spinbutton_adj = gtk_adjustment_new (pref_app.single_frame.step, 0, 1e+08, 1, 10, 10);
    sf_step_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (sf_step_spinbutton_adj), 1, 0);
    gtk_widget_show (sf_step_spinbutton);
    gtk_table_attach (GTK_TABLE (table9), sf_step_spinbutton, 3, 4, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_step_spinbutton, _ ("Increment the counter for frame numbering by the number provided."), NULL);
    
    label60 = gtk_label_new (_ ("Video Settings"));
    gtk_widget_show (label60);
    gtk_frame_set_label_widget (GTK_FRAME (frame3), label60);
    
    single_tab_label = gtk_label_new_with_mnemonic (_ ("S_ingle-Frame"));
    gtk_widget_show (single_tab_label);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (pref_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pref_notebook), 1), single_tab_label);
    
    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox1);
    gtk_container_add (GTK_CONTAINER (pref_notebook), vbox1);
    
    frame1 = gtk_frame_new (NULL);
    gtk_widget_show (frame1);
    gtk_box_pack_start (GTK_BOX (vbox1), frame1, FALSE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (frame1), 4);
    
    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox2);
    gtk_container_add (GTK_CONTAINER (frame1), vbox2);
    
    // multi-frame
    table5 = gtk_table_new (3, 3, FALSE);
    gtk_widget_show (table5);
    gtk_box_pack_start (GTK_BOX (vbox2), table5, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table5), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table5), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table5), 5);
    
    // file name
    mf_file_name_label = gtk_label_new (_ ("File Name:"));
    gtk_widget_show (mf_file_name_label);
    gtk_table_attach (GTK_TABLE (table5), mf_file_name_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_file_name_label), 0, 0.5);

    mf_file_select_button = gtk_button_new_with_mnemonic (_ ("select"));
    gtk_widget_show (mf_file_select_button);
    gtk_table_attach (GTK_TABLE (table5), mf_file_select_button, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_file_select_button, _ ("Click to open a file selector."), NULL);

    mf_file_name_entry = gtk_entry_new ();
    gtk_widget_show (mf_file_name_entry);
    gtk_table_attach (GTK_TABLE (table5), mf_file_name_entry, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_file_name_entry, _ ("Enter a target filename or pattern here."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( mf_file_name_entry ), strdup(pref_app.multi_frame.file) );
    
    // format
    mf_file_format_label = gtk_label_new (_ ("File Format:"));
    gtk_widget_show (mf_file_format_label);
    gtk_table_attach (GTK_TABLE (table5), mf_file_format_label, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_file_format_label), 0, 0.5);

    mf_format_auto_check = gtk_check_button_new_with_mnemonic (_ ("auto"));
    gtk_widget_show (mf_format_auto_check);
    gtk_table_attach (GTK_TABLE (table5), mf_format_auto_check, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_format_auto_check, _ ("Check here for auto-detection of the file format based on the filename."), NULL);

    mf_format_combo = gtk_combo_new ();
    gtk_combo_set_value_in_list (GTK_COMBO (mf_format_combo), TRUE, FALSE);
    g_object_set_data (G_OBJECT (GTK_COMBO (mf_format_combo)->popwin),
                     "GladeParentKey", mf_format_combo);
    {
        int a;
        for (a = (CAP_FFM -1); a < (NUMCAPS -1); a++) {
            mf_format_items = g_list_append ( mf_format_items, format_combo_entries[a] );
        }
    }
    gtk_combo_set_popdown_strings (GTK_COMBO (mf_format_combo), mf_format_items);
    gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(mf_format_combo)->entry), FALSE);
    gtk_widget_show (mf_format_combo);
    gtk_table_attach (GTK_TABLE (table5), mf_format_combo, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_format_combo, _ ("Click and pick a file format from the list of available formats."), NULL);

    if ( pref_app.multi_frame.target == 0 ) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mf_format_auto_check), TRUE);
        gtk_widget_set_sensitive ( mf_format_combo, FALSE );
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mf_format_auto_check), FALSE);
        gtk_widget_set_sensitive ( mf_format_combo, TRUE );
    }
    on_mf_file_name_entry_changed (GTK_ENTRY(mf_file_name_entry), NULL);
    
    // codec
    mf_codec_label = gtk_label_new (_ ("Video Codec:"));
    gtk_widget_show (mf_codec_label);
    gtk_table_attach (GTK_TABLE (table5), mf_codec_label, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_codec_label), 0, 0.5);
    
    mf_codec_auto_check = gtk_check_button_new_with_mnemonic (_ ("auto"));
    gtk_widget_show (mf_codec_auto_check);
    gtk_table_attach (GTK_TABLE (table5), mf_codec_auto_check, 1, 2, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_codec_auto_check, _ ("Check here for auto-detection of the default codec based on the file format."), NULL);

    mf_codec_combo = gtk_combo_new ();
    gtk_combo_set_value_in_list (GTK_COMBO (mf_codec_combo), TRUE, FALSE);
    g_object_set_data (G_OBJECT (GTK_COMBO (mf_codec_combo)->popwin),
                     "GladeParentKey", mf_codec_combo);
    gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(mf_codec_combo)->entry), FALSE);
    gtk_widget_show (mf_codec_combo);
    gtk_table_attach (GTK_TABLE (table5), mf_codec_combo, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_codec_combo, _ ("Click here and pick a codec from the list of codecs available for the selected file format."), NULL);

    if ( pref_app.multi_frame.targetCodec == 0 ) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mf_codec_auto_check), TRUE);
        gtk_widget_set_sensitive ( mf_codec_combo, FALSE );
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mf_codec_auto_check), FALSE);
        gtk_widget_set_sensitive ( mf_codec_combo, TRUE );
    }
    // codec combo contents will be set after setting up the audio codec combo
    
    
    table6 = gtk_table_new (3, 4, FALSE);
    gtk_widget_show (table6);
    gtk_box_pack_start (GTK_BOX (vbox2), table6, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table6), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table6), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table6), 10);

    // fps
    mf_fps_label = gtk_label_new (_ ("Frames per\nSecond:"));
//    gtk_widget_show (mf_fps_label);
    gtk_table_attach (GTK_TABLE (table6), mf_fps_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_fps_label), 0, 0.5);
        
    mf_fps_widget = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new ((pref_app.multi_frame.fps/100), 1, 100, 0.01, 1, 0)));
//    gtk_widget_show (mf_fps_widget);
    gtk_table_attach (GTK_TABLE (table6), mf_fps_widget, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (mf_fps_widget), 2);
    on_mf_codec_combo_changed (GTK_ENTRY(GTK_COMBO(mf_codec_combo)->entry), NULL);
    // set saved codec to smth. we  will never see, so set_mf_fps_widget_from_codec will always double smth. during startup
    mf_fps_widget_save_old_codec = -1;
    set_mf_fps_widget_from_codec(mf_t_codec);
    
    // max time
    mf_max_time_label = gtk_label_new (_ ("Maximum\nTime:"));
    gtk_widget_show (mf_max_time_label);
    gtk_table_attach (GTK_TABLE (table6), mf_max_time_label, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_max_time_label), 0, 0.5);

    mf_max_time_spinbutton_adj = gtk_adjustment_new (pref_app.multi_frame.time, 0, 1e+08, 1, 10, 10);
    mf_max_time_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (mf_max_time_spinbutton_adj), 1, 0);
    gtk_widget_show (mf_max_time_spinbutton);
    gtk_table_attach (GTK_TABLE (table6), mf_max_time_spinbutton, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_max_time_spinbutton, _ ("Set the maximum time to record in number of seconds. A value of zero makes a capture session run until stopped interactively."), NULL);

    // max frames
    mf_max_frames_label = gtk_label_new (_ ("Maximum\nFrames:"));
    gtk_widget_show (mf_max_frames_label);
    gtk_table_attach (GTK_TABLE (table6), mf_max_frames_label, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_max_frames_label), 0, 0.5);

    mf_max_frames_spinbutton_adj = gtk_adjustment_new (pref_app.multi_frame.frames, 0, 1e+08, 1, 10, 10);
    mf_max_frames_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (mf_max_frames_spinbutton_adj), 1, 0);
    gtk_widget_show (mf_max_frames_spinbutton);
    gtk_table_attach (GTK_TABLE (table6), mf_max_frames_spinbutton, 3, 4, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_max_frames_spinbutton, _ ("Set the maximum number of frames to record. A value of zero makes a capture session run until stopped interactively."), NULL);
    
    // compress
    mf_compress_label = gtk_label_new (_ ("Compression\nLevel:"));
    gtk_widget_show (mf_compress_label);
    gtk_table_attach (GTK_TABLE (table6), mf_compress_label, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_compress_label), 0, 0.5);

    mf_compress_hscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (pref_app.multi_frame.compress, 0, 9, 1, 5, 0)));
    gtk_widget_show (mf_compress_hscale);
    gtk_table_attach (GTK_TABLE (table6), mf_compress_hscale, 3, 4, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (mf_compress_hscale), 0);
    
    // quality
    mf_quality_label = gtk_label_new (_ ("Quality:"));
    gtk_widget_show (mf_quality_label);
    gtk_table_attach (GTK_TABLE (table6), mf_quality_label, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_quality_label), 0, 0.5);
    
    mf_quality_hscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (pref_app.multi_frame.quality, 1, 100, 1, 10, 0)));
    gtk_widget_show (mf_quality_hscale);
    gtk_table_attach (GTK_TABLE (table6), mf_quality_hscale, 1, 2, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (mf_quality_hscale), 0);

    
    label43 = gtk_label_new (_ ("Video Settings"));
    gtk_widget_show (label43);
    gtk_frame_set_label_widget (GTK_FRAME (frame1), label43);
    
    frame2 = gtk_frame_new (NULL);
    gtk_widget_show (frame2);
    gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (frame2), 4);
    
    table7 = gtk_table_new (5, 3, FALSE);
    gtk_widget_show (table7);
    gtk_container_add (GTK_CONTAINER (frame2), table7);
    gtk_container_set_border_width (GTK_CONTAINER (table7), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table7), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table7), 10);

    hbox3 = gtk_hbox_new (FALSE, 4);
    gtk_widget_show (hbox3);
    gtk_table_attach (GTK_TABLE (table7), hbox3, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    
    // mf audio settings
    mf_audio_check = gtk_check_button_new_with_mnemonic (_ ("Enable\n_Audio"));
    if ( pref_app.multi_frame.audioWanted > 0 ) 
        gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(mf_audio_check), TRUE);
    else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mf_audio_check), FALSE);
    gtk_widget_show (mf_audio_check);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_check, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_audio_check, _ ("Toggle audio capture on/off."), NULL);
    
    // audio codec
    mf_audio_codec_label = gtk_label_new (_ ("Audio\nCodec:"));
    gtk_widget_show (mf_audio_codec_label);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_codec_label, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_audio_codec_label), 0, 0.5);
    
    hbox4 = gtk_hbox_new (FALSE, 2);
    gtk_widget_show (hbox4);
    gtk_table_attach (GTK_TABLE (table7), hbox4, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);

    mf_audio_codec_auto_check = gtk_check_button_new_with_mnemonic (_ ("auto"));
    gtk_widget_show (mf_audio_codec_auto_check);
    gtk_box_pack_start (GTK_BOX (hbox4), mf_audio_codec_auto_check, FALSE, FALSE, 0);
    gtk_tooltips_set_tip (tooltips, mf_audio_codec_auto_check, _ ("Check here for auto-detection of the default audio codec based on the file format."), NULL);
    
    mf_audio_codec_combo = gtk_combo_new ();
    gtk_combo_set_value_in_list (GTK_COMBO (mf_audio_codec_combo), TRUE, FALSE);
    g_object_set_data (G_OBJECT (GTK_COMBO (mf_audio_codec_combo)->popwin),
                     "GladeParentKey", mf_audio_codec_combo);
    gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(mf_audio_codec_combo)->entry), FALSE);
    gtk_box_pack_start (GTK_BOX (hbox4), mf_audio_codec_combo, FALSE, FALSE, 0);
    gtk_widget_show (mf_audio_codec_combo);
    gtk_tooltips_set_tip (tooltips, mf_audio_codec_combo, _ ("Click here and pick a codec from the list of codecs available for the selected file format."), NULL);
    on_mf_format_combo_changed(GTK_ENTRY(GTK_COMBO(mf_format_combo)->entry), NULL);

    if ( pref_app.multi_frame.au_targetCodec == 0 ) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mf_audio_codec_auto_check), TRUE);
        gtk_widget_set_sensitive ( mf_audio_codec_combo, FALSE );
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mf_audio_codec_auto_check), FALSE);
        gtk_widget_set_sensitive ( mf_audio_codec_combo, TRUE );
    }
    
    // sample rate
    mf_audio_sample_rate_label = gtk_label_new (_ ("Sample\nRate:"));
    gtk_widget_show (mf_audio_sample_rate_label);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_sample_rate_label, 1, 2, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_audio_sample_rate_label), 0, 0.5);
    
    mf_audio_sample_rate_spinbutton_adj = gtk_adjustment_new (pref_app.multi_frame.sndrate, 10000, 1e+08, 50, 1000, 10);
    mf_audio_sample_rate_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (mf_audio_sample_rate_spinbutton_adj), 1, 0);
    gtk_widget_show (mf_audio_sample_rate_spinbutton);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_sample_rate_spinbutton, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_audio_sample_rate_spinbutton, _ ("Set the audio capture sample rate."), NULL);
    
    // bit rate
    mf_audio_bit_rate_label = gtk_label_new (_ ("Bit Rate:"));
    gtk_widget_show (mf_audio_bit_rate_label);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_bit_rate_label, 1, 2, 3, 4,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_audio_bit_rate_label), 0, 0.5);
    
    mf_audio_bit_rate_spinbutton_adj = gtk_adjustment_new (pref_app.multi_frame.sndsize, 10000, 1e+08, 50, 1000, 10);
    mf_audio_bit_rate_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (mf_audio_bit_rate_spinbutton_adj), 1, 0);
    gtk_widget_show (mf_audio_bit_rate_spinbutton);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_bit_rate_spinbutton, 2, 3, 3, 4,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_audio_bit_rate_spinbutton, _ ("Set the audio capture bit rate."), NULL);
    
    // audio in
    mf_audio_in_label = gtk_label_new (_ ("Input\nDevice:"));
    gtk_widget_show (mf_audio_in_label);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_in_label, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_audio_in_label), 0, 0.5); 

    mf_audio_in_select_button = gtk_button_new_with_mnemonic (_ ("select"));
    gtk_widget_show (mf_audio_in_select_button);
    gtk_box_pack_start (GTK_BOX (hbox3), mf_audio_in_select_button, TRUE, TRUE, 0);
    gtk_tooltips_set_tip (tooltips, mf_audio_in_select_button, _ ("Click to open a file selector."), NULL); 
    
    mf_audio_in_entry = gtk_entry_new ();
    gtk_widget_show (mf_audio_in_entry);
    gtk_box_pack_start (GTK_BOX (hbox3), mf_audio_in_entry, TRUE, TRUE, 0);
    gtk_tooltips_set_tip (tooltips, mf_audio_in_entry, _ ("Enter an  audio device name here."), NULL); 
    gtk_entry_set_text (GTK_ENTRY(mf_audio_in_entry), strdup(pref_app.snddev) ); 
    
    // audio channels
    mf_audio_channels_label = gtk_label_new (_ ("Channels:"));
    gtk_widget_show (mf_audio_channels_label);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_channels_label, 1, 2, 4, 5,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_audio_channels_label), 0, 0.5);
    
    mf_audio_channels_hscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (pref_app.multi_frame.sndchannels, 1, 5, 1, 1, 0)));
    gtk_widget_show (mf_audio_channels_hscale);
    gtk_table_attach (GTK_TABLE (table7), mf_audio_channels_hscale, 2, 3, 4, 5,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_FILL), 0, 0);
    gtk_scale_set_digits (GTK_SCALE (mf_audio_channels_hscale), 0);
    
    // toggle sensitivity on/off based on mf_audio_check
    on_mf_audio_check_toggled( GTK_TOGGLE_BUTTON (mf_audio_check), NULL);
    
    label44 = gtk_label_new (_ ("Audio Settings"));
    gtk_widget_show (label44);
    gtk_frame_set_label_widget (GTK_FRAME (frame2), label44);
    
    multi_tab_label = gtk_label_new_with_mnemonic (_ ("M_ulti-Frame"));
    gtk_widget_show (multi_tab_label);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (pref_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pref_notebook), 2), multi_tab_label);
    
    vbox5 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox5);
    gtk_container_add (GTK_CONTAINER (pref_notebook), vbox5);
    
    frame4 = gtk_frame_new (NULL);
    gtk_widget_show (frame4);
    gtk_box_pack_start (GTK_BOX (vbox5), frame4, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (frame4), 4);
    
    hbox1 = gtk_hbox_new (FALSE, 5);
    gtk_widget_show (hbox1);
    gtk_container_add (GTK_CONTAINER (frame4), hbox1);
    gtk_container_set_border_width (GTK_CONTAINER (hbox1), 4);
    
    // commands
    // help command
    cmd_help_label = gtk_label_new (_ ("Help Command:"));
    gtk_widget_show (cmd_help_label);
    gtk_box_pack_start (GTK_BOX (hbox1), cmd_help_label, FALSE, FALSE, 0);

    cmd_help_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (cmd_help_try_button);
    gtk_box_pack_start (GTK_BOX (hbox1), cmd_help_try_button, FALSE, FALSE, 0);
    gtk_tooltips_set_tip (tooltips, cmd_help_try_button, _ ("Click to test the help command."), NULL);

    cmd_help_entry = gtk_entry_new ();
    gtk_widget_show (cmd_help_entry);
    gtk_box_pack_start (GTK_BOX (hbox1), cmd_help_entry, TRUE, TRUE, 0);
    gtk_tooltips_set_tip (tooltips, cmd_help_entry, _ ("Enter the command used to display the help screen."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( cmd_help_entry ), strdup(pref_app.help_cmd) );
    
    label93 = gtk_label_new (_ ("Global Commands"));
    gtk_widget_show (label93);
    gtk_frame_set_label_widget (GTK_FRAME (frame4), label93);
    
    frame5 = gtk_frame_new (NULL);
    gtk_widget_show (frame5);
    gtk_box_pack_start (GTK_BOX (vbox5), frame5, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (frame5), 4);
    
    table12 = gtk_table_new (3, 3, FALSE);
    gtk_widget_show (table12);
    gtk_container_add (GTK_CONTAINER (frame5), table12);
    gtk_container_set_border_width (GTK_CONTAINER (table12), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table12), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table12), 5);
    
    // sf commands
    // sf playback command
    sf_cmd_play_label = gtk_label_new (_ ("Playback Comand:"));
    gtk_widget_show (sf_cmd_play_label);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_play_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_cmd_play_label), 0, 0.5);

    sf_cmd_play_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (sf_cmd_play_try_button);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_play_try_button, 1, 2, 0, 1,
    (GtkAttachOptions) 0,
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_cmd_play_try_button, _ ("Click to test the playback command."), NULL);
    
    sf_cmd_play_entry = gtk_entry_new ();
    gtk_widget_show (sf_cmd_play_entry);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_play_entry, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_cmd_play_entry, _ ("Enter the command to be executed for playing back the recorded video."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( sf_cmd_play_entry ), strdup(pref_app.single_frame.play_cmd) );
    
    // sf encoding command
    sf_cmd_encode_label = gtk_label_new (_ ("Encoding Command:"));
    gtk_widget_show (sf_cmd_encode_label);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_encode_label, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_cmd_encode_label), 0, 0.5);

    sf_cmd_encode_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (sf_cmd_encode_try_button);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_encode_try_button, 1, 2, 1, 2,
    (GtkAttachOptions) 0,
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_cmd_encode_try_button, _ ("Click to test the encoding command."), NULL);

    sf_cmd_encode_entry = gtk_entry_new ();
    gtk_widget_show (sf_cmd_encode_entry);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_encode_entry, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_cmd_encode_entry, _ ("Enter the command to be executed for encoding the recorded individual frames."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( sf_cmd_encode_entry ), strdup(pref_app.single_frame.video_cmd) );
    
    // sf edit command
    sf_cmd_edit_label = gtk_label_new (_ ("Edit Command:"));
    gtk_widget_show (sf_cmd_edit_label);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_edit_label, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (sf_cmd_edit_label), 0, 0.5);

    sf_cmd_edit_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (sf_cmd_edit_try_button);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_edit_try_button, 1, 2, 2, 3,
    (GtkAttachOptions) 0,
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_cmd_edit_try_button, _ ("Click to test the edit command."), NULL);
    
    sf_cmd_edit_entry = gtk_entry_new ();
    gtk_widget_show (sf_cmd_edit_entry);
    gtk_table_attach (GTK_TABLE (table12), sf_cmd_edit_entry, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, sf_cmd_edit_entry, _ ("Enter the command to be executed for editing the current frame."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( sf_cmd_edit_entry ), strdup(pref_app.single_frame.edit_cmd) );
    
    label94 = gtk_label_new (_ ("Single-Frame Capture Commands"));
    gtk_widget_show (label94);
    gtk_frame_set_label_widget (GTK_FRAME (frame5), label94);
    
    frame6 = gtk_frame_new (NULL);
    gtk_widget_show (frame6);
    gtk_box_pack_start (GTK_BOX (vbox5), frame6, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (frame6), 4);
    
    table13 = gtk_table_new (3, 3, FALSE);
    gtk_widget_show (table13);
    gtk_container_add (GTK_CONTAINER (frame6), table13);
    gtk_container_set_border_width (GTK_CONTAINER (table13), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table13), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table13), 5);
    
    // mf commands
    // mf playback command
    mf_cmd_play_label = gtk_label_new (_ ("Playback Command:"));
    gtk_widget_show (mf_cmd_play_label);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_play_label, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_cmd_play_label), 0, 0.5);

    mf_cmd_play_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (mf_cmd_play_try_button);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_play_try_button, 1, 2, 0, 1,
    (GtkAttachOptions) 0,
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_cmd_play_try_button, _ ("Click to test the playback command."), NULL);

    mf_cmd_play_entry = gtk_entry_new ();
    gtk_widget_show (mf_cmd_play_entry);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_play_entry, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_cmd_play_entry, _ ("Enter the command to be executed for playing back the recorded video."), NULL);
    gtk_entry_set_text ( GTK_ENTRY( mf_cmd_play_entry ), strdup(pref_app.multi_frame.play_cmd) );
    
    // mf encoding command
    mf_cmd_encode_label = gtk_label_new (_ ("Transcoding Command:"));
    gtk_widget_show (mf_cmd_encode_label);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_encode_label, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_cmd_encode_label), 0, 0.5);

    mf_cmd_encode_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (mf_cmd_encode_try_button);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_encode_try_button, 1, 2, 1, 2,
    (GtkAttachOptions) 0,
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_cmd_encode_try_button, _ ("Click to test the transcode command."), NULL);

    mf_cmd_encode_entry = gtk_entry_new ();
    gtk_widget_show (mf_cmd_encode_entry);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_encode_entry, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_cmd_encode_entry, _ ("Enter the command to be executed for transcoding the recorded video."), NULL);    
    gtk_entry_set_text ( GTK_ENTRY( mf_cmd_encode_entry ), strdup(pref_app.multi_frame.video_cmd) );
    
    // mf edit command
    mf_cmd_edit_label = gtk_label_new (_ ("Edit Command:"));
    gtk_widget_show (mf_cmd_edit_label);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_edit_label, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (mf_cmd_edit_label), 0, 0.5);
    
    mf_cmd_edit_try_button = gtk_button_new_with_mnemonic (_ ("try"));
    gtk_widget_show (mf_cmd_edit_try_button);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_edit_try_button, 1, 2, 2, 3,
    (GtkAttachOptions) 0,
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_cmd_edit_try_button, _ ("Click to test the edit command."), NULL);

    mf_cmd_edit_entry = gtk_entry_new ();
    gtk_widget_show (mf_cmd_edit_entry);
    gtk_table_attach (GTK_TABLE (table13), mf_cmd_edit_entry, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (0), 0, 0);
    gtk_tooltips_set_tip (tooltips, mf_cmd_edit_entry, _ ("Enter the command to be executed for editing the recorded video."), NULL);    
    gtk_entry_set_text ( GTK_ENTRY( mf_cmd_edit_entry ), strdup(pref_app.multi_frame.edit_cmd) );
    
    label95 = gtk_label_new (_ ("Multi-Frame Capture Commands"));
    gtk_widget_show (label95);
    gtk_frame_set_label_widget (GTK_FRAME (frame6), label95);
    
    label92 = gtk_label_new_with_mnemonic (_ ("C_ommands"));
    gtk_widget_show (label92);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (pref_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pref_notebook), 3), label92);
    
    // action area
    dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
    gtk_widget_show (dialog_action_area1);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
    
    helpbutton1 = gtk_button_new_from_stock ("gtk-help");
    gtk_widget_show (helpbutton1);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), helpbutton1, GTK_RESPONSE_HELP);
    GTK_WIDGET_SET_FLAGS (helpbutton1, GTK_CAN_DEFAULT);
    
    cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
    gtk_widget_show (cancelbutton1);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), cancelbutton1, GTK_RESPONSE_CANCEL);
    GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);
    
    okbutton1 = gtk_button_new_from_stock ("gtk-ok");
    gtk_widget_show (okbutton1);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), okbutton1, GTK_RESPONSE_OK);
    GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);
    
    g_signal_connect ((gpointer) dialog1, "delete_event",
    G_CALLBACK (on_dialog1_delete_event),
    NULL);
    g_signal_connect ((gpointer) dialog1, "destroy",
    G_CALLBACK (on_dialog1_destroy),
    NULL);
    g_signal_connect ((gpointer) dialog1, "close",
    G_CALLBACK (on_dialog1_close),
    NULL);
    g_signal_connect ((gpointer) dialog1, "response",
    G_CALLBACK (on_dialog1_response),
    NULL);
    g_signal_connect ((gpointer) cap_mouse_p_check, "toggled",
    G_CALLBACK (on_cap_mouse_p_check_toggled),
    NULL);
    g_signal_connect ((gpointer) sf_format_auto_check, "toggled",
    G_CALLBACK (on_sf_format_auto_check_toggled),
    NULL);
    g_signal_connect ((gpointer) sf_file_name_entry, "changed",
    G_CALLBACK (on_sf_file_name_entry_changed),
    NULL);
    g_signal_connect ((gpointer) mf_format_auto_check, "toggled",
    G_CALLBACK (on_mf_format_auto_check_toggled),
    NULL);
    g_signal_connect ((gpointer) mf_file_name_entry, "changed",
    G_CALLBACK (on_mf_file_name_entry_changed),
    NULL);
    g_signal_connect ((gpointer) mf_codec_auto_check, "toggled",
    G_CALLBACK (on_mf_codec_auto_check_toggled),
    NULL);
    g_signal_connect ((gpointer) GTK_COMBO(mf_format_combo)->entry, "changed",
    G_CALLBACK (on_mf_format_combo_changed),
    NULL);
    g_signal_connect ((gpointer) mf_audio_check, "toggled",
    G_CALLBACK (on_mf_audio_check_toggled),
    NULL);
    g_signal_connect ((gpointer) GTK_COMBO(mf_codec_combo)->entry, "changed",
    G_CALLBACK (on_mf_codec_combo_changed),
    NULL);
    g_signal_connect ((gpointer) cmd_help_try_button, "clicked",
    G_CALLBACK (on_cmd_help_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) sf_cmd_play_try_button, "clicked",
    G_CALLBACK (on_sf_cmd_play_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) sf_cmd_encode_try_button, "clicked",
    G_CALLBACK (on_sf_cmd_encode_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) sf_cmd_edit_try_button, "clicked",
    G_CALLBACK (on_sf_cmd_edit_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) mf_cmd_play_try_button, "clicked",
    G_CALLBACK (on_mf_cmd_play_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) mf_cmd_encode_try_button, "clicked",
    G_CALLBACK (on_mf_cmd_encode_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) mf_cmd_edit_try_button, "clicked",
    G_CALLBACK (on_mf_cmd_edit_try_button_clicked),
    NULL);
    g_signal_connect ((gpointer) mf_audio_codec_auto_check, "toggled",
    G_CALLBACK (on_mf_audio_codec_auto_check_toggled),
    NULL);
    g_signal_connect ((gpointer) sf_file_select_button, "clicked",
    G_CALLBACK (on_sf_file_select_button_clicked),
    NULL);
    g_signal_connect ((gpointer) mf_file_select_button, "clicked",
    G_CALLBACK (on_mf_file_select_button_clicked),
    NULL);
    g_signal_connect ((gpointer) mf_audio_in_select_button, "clicked",
    G_CALLBACK (on_mf_audio_in_select_button_clicked),
    NULL);

    /*
     #ifdef HAVE_FFMPEG_AUDIO
     g_signal_connect ((gpointer) pref_audio, "toggled",
     G_CALLBACK (on_pref_audio_toggled),
     NULL);
     #endif // HAVE_FFMPEG_AUDIO
     */
    
    
    /*  g_signal_connect ((gpointer) sf_format_item1, "activate",
     G_CALLBACK (on_sf_format_item1_activate),
     NULL);
     */
    
    /* Store pointers to all widgets, for use by lookup_widget(). */
    GLADE_HOOKUP_OBJECT_NO_REF (dialog1, dialog1, "dialog1");
    GLADE_HOOKUP_OBJECT_NO_REF (dialog1, dialog_vbox1, "dialog_vbox1");
    GLADE_HOOKUP_OBJECT (dialog1, pref_notebook, "pref_notebook");
    GLADE_HOOKUP_OBJECT (dialog1, frame7, "frame7");
    GLADE_HOOKUP_OBJECT (dialog1, table14, "table14");
    GLADE_HOOKUP_OBJECT (dialog1, def_capt_label, "def_capt_label");
    GLADE_HOOKUP_OBJECT (dialog1, single_frame_radio, "single_frame_radio");
    GLADE_HOOKUP_OBJECT (dialog1, cap_mouse_p_check, "cap_mouse_p_check");
    GLADE_HOOKUP_OBJECT (dialog1, cap_mouse_p_black_radio, "cap_mouse_p_black_radio");
    GLADE_HOOKUP_OBJECT (dialog1, cap_mouse_p_white_radio, "cap_mouse_p_white_radio");
    GLADE_HOOKUP_OBJECT (dialog1, save_cap_geo_check, "save_cap_geo_check");
    GLADE_HOOKUP_OBJECT (dialog1, use_shm_check, "use_shm_check");
    GLADE_HOOKUP_OBJECT (dialog1, autocont_check, "autocont_check");
    GLADE_HOOKUP_OBJECT (dialog1, multi_frame_radio, "multi_frame_radio");
    GLADE_HOOKUP_OBJECT (dialog1, label103, "label103");
    GLADE_HOOKUP_OBJECT (dialog1, general_tab_label, "general_tab_label");
    GLADE_HOOKUP_OBJECT (dialog1, frame3, "frame3");
    GLADE_HOOKUP_OBJECT (dialog1, vbox3, "vbox3");
    GLADE_HOOKUP_OBJECT (dialog1, table8, "table8");
    GLADE_HOOKUP_OBJECT (dialog1, sf_file_name_label, "sf_file_name_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_file_format_label, "sf_file_format_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_file_select_button, "sf_file_select_button");
    GLADE_HOOKUP_OBJECT (dialog1, sf_file_name_entry, "sf_file_name_entry");
    GLADE_HOOKUP_OBJECT (dialog1, sf_format_auto_check, "sf_format_auto_check");
    GLADE_HOOKUP_OBJECT (dialog1, sf_format_combo, "sf_format_combo");
    GLADE_HOOKUP_OBJECT (dialog1, table9, "table9");
    GLADE_HOOKUP_OBJECT (dialog1, sf_fps_label, "sf_fps_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_max_time_label, "sf_max_time_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_quality_label, "sf_quality_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_fps_widget, "sf_fps_widget");
    GLADE_HOOKUP_OBJECT (dialog1, sf_quality_hscale, "sf_quality_hscale");
    GLADE_HOOKUP_OBJECT (dialog1, sf_max_frames_label, "sf_max_frames_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_compress_label, "sf_compress_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_compress_hscale, "sf_compress_hscale");
    GLADE_HOOKUP_OBJECT (dialog1, sf_max_time_spinbutton, "sf_max_time_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, sf_max_frames_spinbutton, "sf_max_frames_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, sf_start_no_label, "sf_start_no_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_step_label, "sf_step_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_start_no_spinbutton, "sf_start_no_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, sf_step_spinbutton, "sf_step_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, label60, "label60");
    GLADE_HOOKUP_OBJECT (dialog1, single_tab_label, "single_tab_label");
    GLADE_HOOKUP_OBJECT (dialog1, vbox1, "vbox1");
    GLADE_HOOKUP_OBJECT (dialog1, frame1, "frame1");
    GLADE_HOOKUP_OBJECT (dialog1, vbox2, "vbox2");
    GLADE_HOOKUP_OBJECT (dialog1, table5, "table5");
    GLADE_HOOKUP_OBJECT (dialog1, mf_file_name_label, "mf_file_name_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_file_format_label, "mf_file_format_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_codec_label, "mf_codec_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_file_select_button, "mf_file_select_button");
    GLADE_HOOKUP_OBJECT (dialog1, mf_file_name_entry, "mf_file_name_entry");
    GLADE_HOOKUP_OBJECT (dialog1, mf_format_auto_check, "mf_format_auto_check");
    GLADE_HOOKUP_OBJECT (dialog1, mf_codec_auto_check, "mf_codec_auto_check");
    GLADE_HOOKUP_OBJECT (dialog1, mf_format_combo, "mf_format_combo");
    GLADE_HOOKUP_OBJECT (dialog1, mf_codec_combo, "mf_codec_optionmenu");
    GLADE_HOOKUP_OBJECT (dialog1, table6, "table6");
    GLADE_HOOKUP_OBJECT (dialog1, mf_fps_label, "mf_fps_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_max_time_label, "mf_max_time_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_max_frames_label, "mf_max_frames_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_compress_label, "mf_compress_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_quality_label, "mf_quality_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_fps_widget, "mf_fps_widget");
    GLADE_HOOKUP_OBJECT (dialog1, mf_compress_hscale, "mf_compress_hscale");
    GLADE_HOOKUP_OBJECT (dialog1, mf_quality_hscale, "mf_quality_hscale");
    GLADE_HOOKUP_OBJECT (dialog1, mf_max_time_spinbutton, "mf_max_time_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, mf_max_frames_spinbutton, "mf_max_frames_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, label43, "label43");
    GLADE_HOOKUP_OBJECT (dialog1, frame2, "frame2");
    GLADE_HOOKUP_OBJECT (dialog1, table7, "table7");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_check, "mf_audio_check");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_codec_auto_check, "mf_audio_codec_auto_check");
    GLADE_HOOKUP_OBJECT (dialog1, hbox4, "hbox4");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_codec_label, "mf_audio_codec_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_codec_combo, "mf_audio_codec_combo");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_sample_rate_label, "mf_audio_sample_rate_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_sample_rate_spinbutton, "mf_audio_sample_rate_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_bit_rate_label, "mf_audio_bit_rate_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_bit_rate_spinbutton, "mf_audio_bit_rate_spinbutton");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_in_label, "mf_audio_in_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_in_select_button, "mf_audio_in_select_button");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_in_entry, "mf_audio_in_entry");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_channels_label, "mf_audio_channels_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_audio_channels_hscale, "mf_audio_channels_hscale"); 
    GLADE_HOOKUP_OBJECT (dialog1, hbox3, "hbox3");
    GLADE_HOOKUP_OBJECT (dialog1, label44, "label44");
    GLADE_HOOKUP_OBJECT (dialog1, multi_tab_label, "multi_tab_label");
    GLADE_HOOKUP_OBJECT (dialog1, vbox5, "vbox5");
    GLADE_HOOKUP_OBJECT (dialog1, frame4, "frame4");
    GLADE_HOOKUP_OBJECT (dialog1, hbox1, "hbox1");
    GLADE_HOOKUP_OBJECT (dialog1, cmd_help_label, "cmd_help_label");
    GLADE_HOOKUP_OBJECT (dialog1, cmd_help_entry, "cmd_help_entry");
    GLADE_HOOKUP_OBJECT (dialog1, cmd_help_try_button, "cmd_help_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, label93, "label93");
    GLADE_HOOKUP_OBJECT (dialog1, frame5, "frame5");
    GLADE_HOOKUP_OBJECT (dialog1, table12, "table12");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_play_label, "sf_cmd_play_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_encode_label, "sf_cmd_encode_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_edit_label, "sf_cmd_edit_label");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_play_entry, "sf_cmd_play_entry");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_play_try_button, "sf_cmd_play_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_encode_entry, "sf_cmd_encode_entry");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_encode_try_button, "sf_cmd_encode_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_edit_entry, "sf_cmd_edit_entry");
    GLADE_HOOKUP_OBJECT (dialog1, sf_cmd_edit_try_button, "sf_cmd_edit_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, label94, "label94");
    GLADE_HOOKUP_OBJECT (dialog1, frame6, "frame6");
    GLADE_HOOKUP_OBJECT (dialog1, table13, "table13");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_play_label, "mf_cmd_play_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_encode_label, "mf_cmd_encode_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_edit_label, "mf_cmd_edit_label");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_play_entry, "mf_cmd_play_entry");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_play_try_button, "mf_cmd_play_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_encode_entry, "mf_cmd_encode_entry");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_encode_try_button, "mf_cmd_encode_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_edit_entry, "mf_cmd_edit_entry");
    GLADE_HOOKUP_OBJECT (dialog1, mf_cmd_edit_try_button, "mf_cmd_edit_try_button");
    GLADE_HOOKUP_OBJECT (dialog1, label95, "label95");
    GLADE_HOOKUP_OBJECT (dialog1, label92, "label92");
    GLADE_HOOKUP_OBJECT_NO_REF (dialog1, dialog_action_area1, "dialog_action_area1");
    GLADE_HOOKUP_OBJECT (dialog1, helpbutton1, "helpbutton1");
    GLADE_HOOKUP_OBJECT (dialog1, cancelbutton1, "cancelbutton1");
    GLADE_HOOKUP_OBJECT (dialog1, okbutton1, "okbutton1");
    GLADE_HOOKUP_OBJECT_NO_REF (dialog1, tooltips, "tooltips");
    
    return dialog1;
}



